@@ -3,14 +3,15 @@ from __future__ import annotations
 import os
 import sys
 from fractions import Fraction
+from heapq import heappop, heappush
 from os.path import splitext
 from subprocess import run
 from typing import Any
 
 import av
-from av import AudioResampler
+from av import AudioResampler, Codec
 
-from auto_editor.ffwrapper import FFmpeg, FileInfo, initFileInfo
+from auto_editor.ffwrapper import FileInfo, initFileInfo
 from auto_editor.lib.contracts import is_int, is_str
 from auto_editor.make_layers import clipify, make_av, make_timeline
 from auto_editor.output import Ensure, parse_bitrate
@@ -81,6 +82,7 @@ def set_video_codec(
         return codec
 
     if codec == "copy":
+        log.deprecated("The `copy` codec is deprecated. auto-editor always re-encodes")
         if src is None:
             log.error("No input to copy its codec from.")
         if not src.videos:
@@ -88,7 +90,13 @@ def set_video_codec(
         codec = src.videos[0].codec
 
     if ctr.vcodecs is not None and codec not in ctr.vcodecs:
-        log.error(codec_error.format(codec, out_ext))
+        try:
+            cobj = Codec(codec, "w")
+        except av.codec.codec.UnknownCodecError:
+            log.error(f"Unknown encoder: {codec}")
+        # Normalize encoder names
+        if cobj.id not in (Codec(x, "w").id for x in ctr.vcodecs):
+            log.error(codec_error.format(codec, out_ext))
 
     return codec
 
@@ -101,18 +109,16 @@ def set_audio_codec(
             codec = "aac"
         else:
             codec = src.audios[0].codec
-            ctx = av.Codec(codec)
-            if ctx.audio_formats is None:
+            if av.Codec(codec, "w").audio_formats is None:
                 codec = "aac"
         if codec not in ctr.acodecs and ctr.default_aud != "none":
             codec = ctr.default_aud
-        if codec == "mp3float":
-            codec = "mp3"
         if codec is None:
             codec = "aac"
         return codec
 
     if codec == "copy":
+        log.deprecated("The `copy` codec is deprecated. auto-editor always re-encodes")
         if src is None:
             log.error("No input to copy its codec from.")
         if not src.audios:
@@ -120,7 +126,13 @@ def set_audio_codec(
         codec = src.audios[0].codec
 
     if ctr.acodecs is None or codec not in ctr.acodecs:
-        log.error(codec_error.format(codec, out_ext))
+        try:
+            cobj = Codec(codec, "w")
+        except av.codec.codec.UnknownCodecError:
+            log.error(f"Unknown encoder: {codec}")
+        # Normalize encoder names
+        if cobj.id not in (Codec(x, "w").id for x in ctr.acodecs):
+            log.error(codec_error.format(codec, out_ext))
 
     return codec
 
@@ -160,7 +172,7 @@ def parse_export(export: str, log: Log) -> dict[str, Any]:
     log.error(f"'{name}': Export must be [{', '.join([s for s in parsing.keys()])}]")
 
 
-def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
+def edit_media(paths: list[str], args: Args, log: Log) -> None:
     bar = initBar(args.progress)
     tl = None
 
@@ -240,7 +252,7 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
         make_json_timeline(export_ops["api"], output, tl, log)
         return
 
-    if export in ("premiere", "resolve-fcp7"):
+    if export in {"premiere", "resolve-fcp7"}:
         from auto_editor.formats.fcp7 import fcp7_write_xml
 
         is_resolve = export.startswith("resolve")
@@ -294,14 +306,21 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
 
         if ctr.default_aud != "none":
             ensure = Ensure(bar, samplerate, log)
-            audio_paths = make_new_audio(tl, ctr, ensure, args, ffmpeg, bar, log)
+            audio_paths = make_new_audio(tl, ctr, ensure, args, bar, log)
         else:
             audio_paths = []
 
+        # Setup video
+        if ctr.default_vid != "none" and tl.v:
+            vframes = render_av(output, tl, args, log)
+            output_stream = next(vframes)
+        else:
+            output_stream, vframes = None, iter([])
+
         # Setup audio
         if audio_paths:
             try:
-                audio_encoder = av.Codec(args.audio_codec)
+                audio_encoder = av.Codec(args.audio_codec, "w")
             except av.FFmpegError as e:
                 log.error(e)
             if audio_encoder.audio_formats is None:
@@ -343,8 +362,8 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
         for i, sub_path in enumerate(sub_paths):
             subtitle_input = av.open(sub_path)
             subtitle_inputs.append(subtitle_input)
-            subtitle_stream = output.add_stream(
-                template=subtitle_input.streams.subtitles[0]
+            subtitle_stream = output.add_stream_from_template(
+                subtitle_input.streams.subtitles[0]
             )
             if i < len(src.subtitles) and src.subtitles[i].lang is not None:
                 subtitle_stream.metadata["language"] = src.subtitles[i].lang  # type: ignore
@@ -352,19 +371,87 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
             subtitle_streams.append(subtitle_stream)
             sub_gen_frames.append(subtitle_input.demux(subtitles=0))
 
-        # Setup video
-        if ctr.default_vid != "none" and tl.v:
-            vframes = render_av(output, tl, args, bar, log)
-            output_stream = next(vframes)
+        no_color = log.no_color or log.machine
+        encoder_titles = []
+        if output_stream is not None:
+            name = output_stream.codec.canonical_name
+            encoder_titles.append(name if no_color else f"\033[95m{name}")
+        if audio_streams:
+            name = audio_streams[0].codec.canonical_name
+            encoder_titles.append(name if no_color else f"\033[96m{name}")
+        if subtitle_streams:
+            name = subtitle_streams[0].codec.canonical_name
+            encoder_titles.append(name if no_color else f"\033[32m{name}")
+
+        title = f"({os.path.splitext(output_path)[1][1:]}) "
+        if no_color:
+            title += "+".join(encoder_titles)
         else:
-            output_stream, vframes = None, iter([])
+            title += "\033[0m+".join(encoder_titles) + "\033[0m"
+        bar.start(tl.end, title)
+
+        MAX_AUDIO_AHEAD = 30  # In timebase, how far audio can be ahead of video.
+        MAX_SUB_AHEAD = 30
+
+        class Priority:
+            __slots__ = ("index", "frame_type", "frame", "stream")
+
+            def __init__(self, value: int | Fraction, frame, stream):
+                self.frame_type: str = stream.type
+                assert self.frame_type in ("audio", "subtitle", "video")
+                if self.frame_type in {"audio", "subtitle"}:
+                    self.index: int | float = round(value * frame.time_base * tl.tb)
+                else:
+                    self.index = float("inf") if value is None else int(value)
+                self.frame = frame
+                self.stream = stream
+
+            def __lt__(self, other):
+                return self.index < other.index
+
+            def __eq__(self, other):
+                return self.index == other.index
+
+        # Priority queue for ordered frames by time_base.
+        frame_queue: list[Priority] = []
+        latest_audio_index = float("-inf")
+        latest_sub_index = float("-inf")
+        earliest_video_index = None
 
-        # Process frames
         while True:
-            audio_frames = [next(frames, None) for frames in audio_gen_frames]
-            video_frame = next(vframes, None)
-            subtitle_frames = [next(packet, None) for packet in sub_gen_frames]
+            if earliest_video_index is None:
+                should_get_audio = True
+                should_get_sub = True
+            else:
+                for item in frame_queue:
+                    if item.frame_type == "audio":
+                        latest_audio_index = max(latest_audio_index, item.index)
+                    elif item.frame_type == "subtitle":
+                        latest_sub_index = max(latest_sub_index, item.index)
 
+                should_get_audio = (
+                    latest_audio_index <= earliest_video_index + MAX_AUDIO_AHEAD
+                )
+                should_get_sub = (
+                    latest_sub_index <= earliest_video_index + MAX_SUB_AHEAD
+                )
+
+            index, video_frame = next(vframes, (0, None))
+
+            if video_frame:
+                earliest_video_index = index
+                heappush(frame_queue, Priority(index, video_frame, output_stream))
+
+            if should_get_audio:
+                audio_frames = [next(frames, None) for frames in audio_gen_frames]
+            else:
+                audio_frames = [None]
+            if should_get_sub:
+                subtitle_frames = [next(packet, None) for packet in sub_gen_frames]
+            else:
+                subtitle_frames = [None]
+
+            # Break if no more frames
             if (
                 all(frame is None for frame in audio_frames)
                 and video_frame is None
@@ -372,34 +459,48 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
             ):
                 break
 
-            for audio_stream, audio_frame in zip(audio_streams, audio_frames):
-                if audio_frame:
+            if should_get_audio:
+                for audio_stream, audio_frame in zip(audio_streams, audio_frames):
                     for reframe in resampler.resample(audio_frame):
-                        output.mux(audio_stream.encode(reframe))
+                        assert reframe.pts is not None
+                        heappush(
+                            frame_queue,
+                            Priority(reframe.pts, reframe, audio_stream),
+                        )
+            if should_get_sub:
+                for subtitle_stream, packet in zip(subtitle_streams, subtitle_frames):
+                    if packet and packet.pts is not None:
+                        packet.stream = subtitle_stream
+                        heappush(
+                            frame_queue, Priority(packet.pts, packet, subtitle_stream)
+                        )
 
-            for subtitle_stream, packet in zip(subtitle_streams, subtitle_frames):
-                if not packet or packet.dts is None:
-                    continue
-                packet.stream = subtitle_stream
-                output.mux(packet)
-
-            if video_frame:
+            while frame_queue and frame_queue[0].index <= index:
+                item = heappop(frame_queue)
+                frame_type = item.frame_type
                 try:
-                    output.mux(output_stream.encode(video_frame))
+                    if frame_type in {"video", "audio"}:
+                        output.mux(item.stream.encode(item.frame))
+                    elif frame_type == "subtitle":
+                        output.mux(item.frame)
                 except av.error.ExternalError:
                     log.error(
-                        f"Generic error for encoder: {output_stream.name}\n"
-                        "Perhaps video quality settings are too low?"
+                        f"Generic error for encoder: {item.stream.name}\n"
+                        f"at {item.index} time_base\nPerhaps video quality settings are too low?"
                     )
                 except av.FFmpegError as e:
                     log.error(e)
 
+            bar.tick(index)
+
         # Flush streams
         if output_stream is not None:
             output.mux(output_stream.encode(None))
         for audio_stream in audio_streams:
             output.mux(audio_stream.encode(None))
 
+        bar.end()
+
         # Close resources
         for audio_input in audio_inputs:
             audio_input.close()
@@ -449,7 +550,7 @@ def edit_media(paths: list[str], ffmpeg: FFmpeg, args: Args, log: Log) -> None:
 
     log.stop_timer()
 
-    if not args.no_open and export in ("default", "audio"):
+    if not args.no_open and export in {"default", "audio"}:
         if args.player is None:
             if sys.platform == "win32":
                 try:
