@@ -139,7 +139,7 @@ def make_timeline(
 
     for i, src in enumerate(sources):
         try:
-            parser = Parser(Lexer("`--edit`", args.edit_based_on))
+            parser = Parser(Lexer("`--edit`", args.edit))
             if log.is_debug:
                 log.debug(f"edit: {parser}")
 
@@ -169,6 +169,8 @@ def make_timeline(
         has_loud = concat((has_loud, result))
         src_index = concat((src_index, np.full(len(result), i, dtype=np.int32)))
 
+    assert len(has_loud) > 0
+
     # Setup for handling custom speeds
     speed_index = has_loud.astype(np.uint)
     speed_map = [args.silent_speed, args.video_speed]
