@@ -104,7 +104,7 @@ def frame_rate(val: str) -> Fraction:
 
 def sample_rate(val: str) -> int:
     num, unit = _split_num_str(val)
-    if unit in ("kHz", "KHz"):
+    if unit in {"kHz", "KHz"}:
         return natural(num * 1000)
     _unit_check(unit, ("", "Hz"))
     return natural(num)
@@ -122,9 +122,9 @@ def time(val: str, tb: Fraction) -> int:
         raise CoerceError(f"'{val}': Invalid time format")
 
     num, unit = _split_num_str(val)
-    if unit in ("s", "sec", "secs", "second", "seconds"):
+    if unit in {"s", "sec", "secs", "second", "seconds"}:
         return round(num * tb)
-    if unit in ("min", "mins", "minute", "minutes"):
+    if unit in {"min", "mins", "minute", "minutes"}:
         return round(num * tb * 60)
     if unit == "hour":
         return round(num * tb * 3600)
@@ -198,6 +198,7 @@ class Args:
     video_codec: str = "auto"
     audio_codec: str = "auto"
     video_bitrate: str = "auto"
+    vprofile: str | None = None
     audio_bitrate: str = "auto"
     scale: float = 1.0
     sn: bool = False
@@ -210,14 +211,13 @@ class Args:
     sample_rate: int | None = None
     resolution: tuple[int, int] | None = None
     background: str = "#000000"
-    edit_based_on: str = "audio"
+    edit: str = "audio"
     keep_tracks_separate: bool = False
     audio_normalize: str = "#f"
     export: str | None = None
     player: str | None = None
     no_open: bool = False
     temp_dir: str | None = None
-    ffmpeg_location: str | None = None
     progress: str = "modern"
     version: bool = False
     debug: bool = False
